var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import { createSelector, firebaseSelector, formValueSelector, _get, _has, _filter, _isString, _orderBy, _map, _mapValues, _omit, _isNil, populate } from '../../dependencies';

import constants from '../constants';
import * as paths from '../firebase/paths';

var userIdFromPropsSelector = function userIdFromPropsSelector(state, props) {
    return props.userId;
};
var conversationIdFromPropsSelector = function conversationIdFromPropsSelector(state, props) {
    return props.conversationId;
};

export var chatPagingSelector = function chatPagingSelector(state) {
    return state.chat;
};

var firebaseUsersSelector = createSelector(firebaseSelector, function (firebase) {
    return _get(firebase, 'data.' + paths.users().store, {});
});

/**
 * Conversations list
 */
export var isUserConversationsLoaded = createSelector(userIdFromPropsSelector, firebaseSelector, function (userId, firebase) {
    return !_get(firebase, ['requesting', paths.userConversations(userId).id]);
});

export var userConversationsSelector = createSelector(userIdFromPropsSelector, firebaseSelector, isUserConversationsLoaded, function (userId, firebase, isLoaded) {
    var userConversations = populate(firebase, paths.userConversations(userId).store, paths.populates.conversations);

    var activeUserConversations = _filter(userConversations, function (userConversation) {
        return _has(userConversation, 'conversation');
    }).map(function (userConversation) {
        var conversation = userConversation.conversation;


        return _extends({
            conversationId: _isString(conversation) ? conversation : conversation.id
        }, userConversation);
    });

    return {
        isLoaded: isLoaded,
        conversations: activeUserConversations
    };
});

var hasUnseenMessages = function hasUnseenMessages(conversation) {
    return Boolean(_get(conversation, 'unseenMessagesCount', 0) > 0);
};

var isUnseenMoreRecent = function isUnseenMoreRecent(conversation) {
    return _get(conversation, ['lastMessage', 'time'], 0);
};

export var conversationsSelector = createSelector(userConversationsSelector, firebaseSelector, function (userConversations, firebase) {
    var conversations = userConversations.conversations.map(function (_ref) {
        var conversationId = _ref.conversationId,
            unseenMessagesCount = _ref.unseenMessagesCount,
            conversation = _ref.conversation;
        return {
            conversationId: conversationId,
            lastMessage: _get(conversation, 'lastMessage'),
            unseenMessagesCount: unseenMessagesCount,
            users: _get(conversation, 'users')
        };
    });

    conversations = _orderBy(conversations, [hasUnseenMessages, isUnseenMoreRecent], ['desc', 'desc']);

    var isEveryConversationLoaded = userConversations.conversations.every(function (_ref2) {
        var conversationId = _ref2.conversationId;

        return !_get(firebase, ['requesting', paths.conversation(conversationId).id]);
    });

    return {
        isEmpty: conversations.legth === 0,
        isLoaded: userConversations.isLoaded && isEveryConversationLoaded,
        conversations: conversations
    };
});

export var unseenConversationsSelector = createSelector(userConversationsSelector, function (userConversations) {
    return userConversations.conversations.filter(hasUnseenMessages);
});

var conversationsSearchFormSelector = formValueSelector(constants.SEARCH_CONVERSATION_FORM_ID);

export var conversationsSearchValueSelector = function conversationsSearchValueSelector(state) {
    return conversationsSearchFormSelector(state, 'text');
};

var isConversationMatchedByFilter = function isConversationMatchedByFilter(_ref3, filterValue) {
    var user = _ref3.user;

    var userName = _get(user, 'name');
    return _isString(userName) && userName.toLowerCase().includes(filterValue.toLowerCase());
};

export var conversationsListSelector = createSelector(conversationsSelector, firebaseUsersSelector, firebaseSelector, conversationsSearchValueSelector, function (_ref4, users, firebase, searchFormValue) {
    var conversations = _ref4.conversations,
        isLoaded = _ref4.isLoaded;

    var conversationsList = conversations.map(function (conversation) {
        var conversationId = conversation.conversationId;

        var conversationDetail = _get(firebase, 'data.' + paths.conversation(conversationId).store, {});

        var conversationUsers = _get(conversationDetail, 'users', {});

        return _extends({}, conversation, conversationDetail, {
            users: _mapValues(conversationUsers, function (_, id) {
                return _get(users, id, id);
            })
        });
    });

    if (searchFormValue) {
        return conversationsList.filter(function (conversation) {
            return isConversationMatchedByFilter(conversation, searchFormValue);
        });
    }

    return {
        conversations: conversationsList,
        isLoaded: isLoaded
    };
});

/**
 * Conversation & Messages
 */
export var isConversationLoadedSelector = createSelector(conversationIdFromPropsSelector, firebaseSelector, function (conversationId, firebase) {
    return !_get(firebase, ['requesting', paths.conversationMessages(conversationId).id]);
});

export var conversationSelector = createSelector(conversationIdFromPropsSelector, conversationsListSelector, function (conversationId, _ref5) {
    var conversations = _ref5.conversations;

    var conversation = conversations.find(function (c) {
        return c.conversationId === conversationId;
    });

    return {
        isEmpty: false,
        isLoaded: Boolean(conversation),
        conversation: _extends({}, conversation, {
            id: conversationId
        })
    };
});

var disabledPagingFromProps = function disabledPagingFromProps(_, _ref6) {
    var disabledPaging = _ref6.disabledPaging;
    return Boolean(disabledPaging);
};

export var messagesCountsSelector = createSelector(chatPagingSelector, conversationSelector, disabledPagingFromProps, function (paging, _ref7, disabledPaging) {
    var _ref7$conversation = _ref7.conversation,
        unseenMessagesCount = _ref7$conversation.unseenMessagesCount,
        messagesCount = _ref7$conversation.messagesCount;
    var page = paging.page,
        amount = paging.amount,
        sentMessages = paging.sentMessages;

    var count = disabledPaging ? Infinity : page * amount + sentMessages;

    return {
        count: unseenMessagesCount > count ? unseenMessagesCount : count,
        sentMessagesCount: sentMessages,
        unseenMessagesCount: unseenMessagesCount,
        messagesCount: messagesCount
    };
});

export var messagesSelector = createSelector(userIdFromPropsSelector, conversationIdFromPropsSelector, firebaseSelector, isConversationLoadedSelector, function (state) {
    return state.chat.firstUnreadMessageId;
}, function (userId, conversationId, firebase, isLoaded, firstUnreadMessageId) {
    var messages = populate(firebase, paths.conversationMessages(conversationId).store, paths.populates.messages);

    var messagesWithId = _map(messages, function (message, key) {
        return _extends({}, message, {
            messageId: key,
            seen: !_get(message, ['unseen', userId], false),
            amIAuthor: _get(message, 'author.authorId', '') === userId
        });
    });

    var firstUnreadMessageIndex = messagesWithId.findIndex(function (_ref8) {
        var messageId = _ref8.messageId;
        return messageId === firstUnreadMessageId;
    });
    var readMessagesCount = Math.max(firstUnreadMessageIndex, _isNil(firstUnreadMessageId) ? messagesWithId.length : 0);

    return {
        isLoaded: isLoaded,
        isEmpty: messagesWithId.length === 0,
        messages: messagesWithId,
        readMessagesCount: readMessagesCount
    };
});

export var unseenSelector = createSelector(userIdFromPropsSelector, conversationIdFromPropsSelector, firebaseSelector, function (userId, conversationId, firebase) {
    var users = _get(firebase, 'data.' + paths.conversation(conversationId).store + '.users', {});

    return _omit(users, userId);
});