const path = require('path');
const Promise = require('bluebird');
const chalk = require('chalk');
const Markdown = require('markdown-to-html').Markdown;
const readdir = require('recursive-readdir');
const fs = Promise.promisifyAll(require('fs'));
const mkdirp = Promise.promisify(require('mkdirp'));

const rootDir = path.resolve(__dirname, '../');
const outputDir = path.resolve(rootDir, 'build');

const indexFile = 'README.md';
const docsDir = path.resolve(rootDir, 'docs');

const htmlTemplate = fs.readFileSync(path.resolve(__dirname, 'assets/template.html'));

const opts = {
};

/* eslint-disable no-console */
const convertFile = (filePath) => {
    return new Promise((resolve, reject) => {
        let converted = '';
        const md = new Markdown();
        const errors = [];

        md.bufmax = 2048;

        md.once('end', () => {
            errors.length ? reject(errors) : resolve(converted);
        });

        md.render(filePath, opts, (err) => {
            if (err) {
                errors.push(err);
            } else {
                md.on('data', chunk => {
                    converted += chunk.toString();
                });
            }
        });
    });
};

const getCompleteHtml = (htmlContent, filename) => {
    // fix links, they should refer to html, not md file
    htmlContent = htmlContent.replace(/\.md">/g, '.html">');

    return htmlTemplate.toString()
        .replace('{markdown}', htmlContent)
        .replace('{baseDir}', path.relative(path.dirname(filename), outputDir) || '.');
};

const createHtmlForMd = async(filename, newName) => {
    try {
        const htmlContent = await convertFile(path.resolve(rootDir, filename));
        const htmlFilename = filename.replace(rootDir, '').replace('.md', '.html');
        let outputPath = path.join(outputDir, htmlFilename);

        if (newName) {
            outputPath = outputPath.replace(/\/([^/]*)\.html$/, `/${newName}.html`);
        }

        await mkdirp(path.dirname(outputPath));
        await fs.writeFileAsync(outputPath, getCompleteHtml(htmlContent, outputPath));

        console.log(`${chalk.green('✓')} File ${filename} converted`);
    } catch (err) {
        console.log(`${chalk.red('✗')} Failed convert of ${filename}`, err);
    }
};

const copyAsset = async(assetPath, outputAssetFilename, assetName) => {
    try {
        const assetFilename = path.basename(assetPath);

        assetName = assetName || assetFilename;
        outputAssetFilename = outputAssetFilename || path.resolve(outputDir, assetFilename);

        const content = await fs.readFileAsync(path.resolve(rootDir, assetPath));

        await fs.writeFileAsync(path.resolve(outputDir, outputAssetFilename), content);
        console.log(`${chalk.green('✓')} ${assetName} copied`);
    } catch (err) {
        console.log(`${chalk.red('✗')} Failed copying ${assetName}`, err);
    }
};

const copyStyles = async() => {
    await copyAsset(
        'node_modules/github-markdown-css/github-markdown.css',
        path.resolve(outputDir, 'github-style.css'),
        'Github CSS styles',
    );
};

const copyImages = async() => {
    await copyAsset(
        path.resolve(docsDir, 'alert-icon.png'),
        path.resolve(outputDir, 'docs', 'alert-icon.png'),
        'Alert icon'
    );
};

const generateSite = async() => {
    try {
        const files = await readdir(docsDir, ['*.png']);

        await Promise.map(files, createHtmlForMd);
        await createHtmlForMd(indexFile, 'index');
        await copyStyles();
        await copyImages();

        console.log(chalk.green('Site completly generated'));
    } catch (err) {
        console.log(chalk.red(`Unexpected error occured ${err}`));
    }
};

generateSite();
